import { isValidElement } from "react";
import { chunkDelimiters, desktopChunkSize, lineDelimiter, minChunkSize, mobileChunkSize, sanitizeRegex, sentenceDelimiters, spaceDelimiter, spaceDelimiters, specialSymbol, symbolMapping, utterancePropertiesAndEvents, } from "./constants.js";
import { setState } from "./state.js";
export function ArrayToText(node) {
    if (typeof node === "string")
        return node;
    return node.map(ArrayToText).join(spaceDelimiter) + spaceDelimiter;
}
export function JSXToArray(node) {
    if (isValidElement(node)) {
        const { children } = node.props;
        if (Array.isArray(children))
            return children.map(JSXToArray);
        return JSXToArray(children);
    }
    return typeof node === "string" ? node : typeof node === "number" ? String(node) : "";
}
export function TextToChunks(text, size) {
    size = size ? Math.max(size, minChunkSize) : isMobile() ? mobileChunkSize : desktopChunkSize;
    const length = text.length;
    const result = [];
    let startIndex = 0;
    while (startIndex < length) {
        let endIndex = Math.min(startIndex + size, length);
        if (endIndex < length && !spaceDelimiters.includes(text[endIndex]))
            for (const delimiter of chunkDelimiters) {
                let delimiterIndex = text.lastIndexOf(delimiter, endIndex) + delimiter.length - 1;
                if (delimiterIndex > startIndex) {
                    endIndex = delimiterIndex;
                    break;
                }
            }
        result.push(text.slice(startIndex, endIndex));
        startIndex = endIndex;
    }
    return result;
}
export function cancel() {
    var _a;
    if (typeof window === "undefined")
        return;
    setState({ stopReason: "manual" });
    (_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.cancel();
}
export function cloneUtterance(utterance, text) {
    const newUtterance = new SpeechSynthesisUtterance(text);
    utterancePropertiesAndEvents.forEach((property) => (newUtterance[property] = utterance[property]));
    return newUtterance;
}
export function findCharIndex(words, index) {
    let currentIndex = 0;
    function recursiveSearch(stringArray, parentIndex = "") {
        if (typeof stringArray === "string") {
            const elementIndex = index - currentIndex;
            return (currentIndex += stringArray.length + 1) > index ? getIndex(parentIndex, elementIndex) : "";
        }
        for (let i = 0; i < stringArray.length; i++) {
            const node = stringArray[i];
            const result = recursiveSearch(node, i);
            if (result)
                return getIndex(parentIndex, result);
        }
        currentIndex++;
        return "";
    }
    return recursiveSearch(words);
}
export const getIndex = (parentIndex, index) => `${parentIndex === "" ? "" : parentIndex + "-"}${index}`;
export function isMobile(iOS = true) {
    var _a;
    let result = (_a = navigator.userAgentData) === null || _a === void 0 ? void 0 : _a.mobile;
    result !== null && result !== void 0 ? result : (result = /Android|webOS|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) || (iOS && /iPhone|iPad|iPod/i.test(navigator.userAgent)));
    return result;
}
export function isParent(parentIndex, index) {
    if (!(index === null || index === void 0 ? void 0 : index.startsWith(parentIndex)))
        return false;
    if (parentIndex) {
        const indexParts = index.split("-");
        const parentIndexParts = parentIndex.split("-");
        for (let i = 0; i < parentIndexParts.length; i++) {
            if (indexParts[i] !== parentIndexParts[i])
                return false;
        }
    }
    return true;
}
export function parent(index) {
    if (!index)
        return "";
    const lastIndex = index.lastIndexOf("-");
    return lastIndex === -1 ? "" : index.slice(0, lastIndex);
}
export const sanitize = (text) => text.replace(sanitizeRegex, (match, group) => (group ? group + ")" : ` ${symbolMapping[match]}${specialSymbol}`));
export function shouldHighlightNextPart(highlightMode, name, utterance, charIndex, isSpecialSymbol) {
    if (name === "word" && (highlightMode === "word" || !charIndex))
        return true;
    const text = utterance.text.slice(0, charIndex).replace(/[ \t]+$/, spaceDelimiter);
    if (highlightMode === "sentence" && (text.at(-isSpecialSymbol - 1) === lineDelimiter || sentenceDelimiters.includes(text.at(-2))))
        return true;
    if (highlightMode === "line" && text.at(-isSpecialSymbol - 1) === lineDelimiter)
        return true;
    if (highlightMode === "paragraph" && name === "sentence")
        return true;
    return false;
}
export function splitNode(highlightMode, node, speakingWord) {
    const { index, length } = speakingWord;
    const beforeIndex = +index.split("-").at(-1);
    const before = node.slice(0, beforeIndex);
    if (highlightMode === "word")
        return [before, node.slice(beforeIndex, beforeIndex + length), node.slice(beforeIndex + length)];
    node = node.slice(beforeIndex);
    const regex = highlightMode === "sentence" ? /(.*?)(\n|[.!?]\s)(.*)/ : /(.*?)(\n)(.*)/;
    const match = node.match(regex);
    if (!match)
        return [before, node, ""];
    const sentence = match[1] + match[2].trimEnd();
    return [before, sentence, node.slice(sentence.length)];
}
